<?php
/***************************************************************************
 *                             ezboard-parse.php
 *                            -------------------
 *   $Id: ezboard-parse.php,v 1.21 2005/01/25 23:56:52 draknek Exp $
 *   begin                : Wednesday, 20 August 2002
 *   author               : Rob Bellaart
 *   email                : bellaart@ghanaweb.com
 *
 *   modified             : Monday, 13 January 2003
 *   author               : Chris Burke
 *   email                : serotonin@earthlink.net
 *
 *   modified             : Monday, 07 July 2003
 *   author               : Jamie Tatum
 *   email                : jt@killsdead.com
 *
 *   modified             : Tuesday, 20 July 2004
 *   author               : Alan Hazelden (Draknek)
 *   email                : ezconvert@draknek.org
 *
 *   purpose              : parsing routines to convert an ezboard forum
 *
 ***************************************************************************/

/***************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 ***************************************************************************/

// Will log in to ezBoard using cURL or fsockopen()
function admin_login ($return = false)
{
    global $config;

    $postvars = 'login=' . urlencode($config['ezboard_user']) . '&password=' . urlencode($config['ezboard_pass']);

    if ($config['connect_with'] == 'curl')
    {
        notice('Logging in to ezboard using cURL...');

        $ch = curl_init ($config['login_page']);

        curl_setopt ($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt ($ch, CURLOPT_HEADER, 1);
        curl_setopt ($ch, CURLOPT_USERAGENT, $config['user_agent']);
        curl_setopt ($ch, CURLOPT_POST, 1);
        curl_setopt ($ch, CURLOPT_POSTFIELDS, $postvars);

        $data = curl_exec ($ch);

        if (!$data)
        {
            fatal_error('Problem with cURL - ' . curl_error($ch));
        }

        curl_close($ch);
    }
    elseif ($config['connect_with'] == 'fsockopen')
    {
		notice('Logging in to ezboard using fsockopen()...');

		$info = parse_url($config['login_page']);
		$request  = 'POST ' . $info['path'] . ($info['query'] ? '?'.$info['query'] : '') . " HTTP/1.0\r\n".
					'Host: ' . $info['host'] . "\r\n".
					'User-Agent: ' . $config['user_agent'] . "\r\n".
					"Content-Type: application/x-www-form-urlencoded\r\n".
					'Content-Length: ' . strlen($postvars) . "\r\n".
					"\r\n$postvars";

		$fid = @fsockopen($info['host'],80);
		if (!$fid)
		{
			fatal_error('Could not connect to ezBoard');
		}
		fputs($fid, $request);

		$data = '';
		while (!feof($fid)) {$data .= fread($fid,8192);}
		fclose($fid);
    }
    else
    {
        fatal_error('Only cURL or fsockopen() are currently supported for logging into ezBoard');
    }

	list($headers,$html) = explode("\r\n\r\n", $data, 2);

	if (preg_match('/.*Set-Cookie: (ezbsession=[^\s;]+)/s',$headers,$match))
	{
		  $cookie = $match[1];
	}
	else
	{
		// panic
	}

	$config['ezboard_cookie'] = $cookie;
	// Really hacky code - ezBoard's cookie is longer than our textfield, so we use two fields
	$cookie1 = substr($cookie,0,255);
	$cookie2 = substr($cookie,255);
	$sql = 'UPDATE ' . EZCONVERT_CONFIG_TABLE . " SET config_value='$cookie1' WHERE config_name='ezboard_cookie'";
	exec_sql($sql);
	$sql = 'UPDATE ' . EZCONVERT_CONFIG_TABLE . " SET config_value='$cookie2' WHERE config_name='ezboard_cookie2'";
	exec_sql($sql);

	if ($return) {return $headers;}
}

// reads the contents of an ezboard page into a buffer
// Modified JT - optionally use cURL to retrieve pages with cookies and other good stuff
function open_page ($arg, &$buffer, $die = true)
{
    global $config;

    $fname = $config['ezboard_forum'].$arg;

    if ($config['connect_with'] == 'curl')
    {
        notice('Opening ezBoard page: <a href="' . $fname . '">' . $fname . '</a> with cURL');

        $ch = curl_init ($fname);

        if ($config['use_admin'])
        {
            curl_setopt ($ch, CURLOPT_COOKIE, $config['ezboard_cookie']);
        }

        curl_setopt ($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt ($ch, CURLOPT_USERAGENT, $config['user_agent']);

        $f = curl_exec ($ch);
        if ($f === false)
        {
           if ($die)
           {
               fatal_error ('Problem with cURL - ' . curl_error($ch));
           }
           else
           {
               warning('Problem with cURL - ' . curl_error($ch));
               return;
           }
        }

        if ($config['strip_crlf'])
        {
		   // Removes every \n and \r character.
           $f = str_replace ( array("\r", "\n"), '', $f );
        }
        $buffer = $f;
    }
    elseif ($config['connect_with'] == 'fsockopen')
    {
        notice('Opening ezBoard page: <a href="' . $fname . '">' . $fname . '</a> with fsockopen()');

        $info = parse_url($fname);

        $request  = 'GET ' . $info['path'] . ($info['query'] ? '?'.$info['query'] : '') . " HTTP/1.0\r\n".
        			'Host: ' . $info['host'] . "\r\n".
					'User-Agent: ' . $config['user_agent'] . "\r\n";
        if ($config['use_admin'])
        {
            $cookie = $config['ezboard_cookie'];
            $request .= "Cookie: $cookie\r\n";
        }
        $request .= "\r\n";

        $fid = @fsockopen($info['host'],80);
        if (!$fid)
        {
            if ($die) {fatal_error($fname . ' could not be opened');}
            else      {warning($fname . ' could not be opened'); return;}
        }
        fputs($fid, $request);

        $buffer = '';
        while (!feof($fid)) {$buffer .= fread($fid,8192);}
        fclose($fid);

        if ($config['strip_crlf'])
        {
           // Removes every \n and \r character.
           $f = str_replace ( array("\r", "\n"), '', $f );
        }
    }
    else
    {
       notice('Opening ezBoard page: <a href="' . $fname . '">' . $fname . '</a> with fopen()');
       $f = fopen ($fname, 'r');
       if ($f === false)
       {
           if ($die)
           {
               fatal_error ($fname . ' could not be opened');
           }
           else
           {
               warning($fname . ' could not be opened');
               return;
           }
       }
       $buffer = "";
       while ($line=fgets($f,5000))
       {
           if ($config['strip_crlf'])
           {
               // Removes every \n and \r character.
			   $f = str_replace ( array("\r", "\n"), '', $f );
           }
           $buffer .= $line;
       }
       fclose ($f);

       if (empty($buffer))
       {
           if ($die)
           {
               fatal_error ('empty data stream returned from ' . $fname);
           }
           else
           {
               warning('empty data stream returned from ' . $fname);
               return;
           }
       }

    }

}

// handles an invalid syntax in an ezboard page
function syntax_error ($tag, $pos, $message)
{
    fatal_error ('Tag ' . htmlspecialchars($tag) . " not found after position $pos.<br />" . htmlspecialchars($message));
}

// retrieves the string between the two tags
// Modified to be case insensitive since some tags are upper
// or lower case....Chris Burke, serotonin@earthlink.net
function retrieve_value ($buffer, $start_tag, $end_tag, &$p)
{
	// Ammended by Draknek to be less clunky
	$lc_buffer = strtolower($buffer);
	$lc_start_tag = strtolower($start_tag);
	$lc_end_tag = strtolower($end_tag);

	$p1 = find_tag($lc_buffer, $lc_start_tag, $p, "Starting tag of $start_tag and $end_tag");
    $p2 = find_tag($lc_buffer, $lc_end_tag, $p, "Ending tag of $start_tag and $end_tag");

	return (substr($buffer, $p1+strlen($start_tag), $p2-$p1-strlen($start_tag)));
}

// finds a tag in the page
function find_tag ($buffer, $tag, &$p, $mandatory)
{
	$pos = strpos($buffer, $tag, $p);
	if ($pos)
	{
		$p = $pos;
		return $pos;
	}
	else
	{
		if (empty($mandatory))
		{
			return false;
		}
		else
		{
			syntax_error ($tag, $p, $mandatory);
		}
	}
}

function unhtml ($string)
{
	return trim(preg_replace('/(&nbsp;|\s)+/', ' ', strip_tags($string)));
}

// parses an EZBOARD webpage with topics
// if EZBOARD changes its html then this function might need to be recoded.
//
// Modified by Chris Burke (serotonin@earthlink.net) to compensate for
// problem introduced when case-sensitivity bug was fixed (made retrieve_value
// case insensitive so it would properly parse certain ezboard HTML)

function parse_topic_list ($page)
{
    global $db, $config;

    notice("Parsing topic list for page " . $page);
    open_page ("?page=$page", $buffer);

	// Get past any sidebars, etc.
	$p = 0;
	find_tag($buffer, '>Topic</font>', $p, '');
	if ($p == 0)
	{
		warning('Heading not found - could be non-english language setting');
	}

	// It's unlikely, but possible to have # of replies hidden, so we check
	$can_see_replies = find_tag($buffer, '>Replies</font>', $p, '');

	// We don't know whether page views information is there
	$can_see_views = find_tag($buffer, '>Views</font>', $p, '');

	// There might be screwy stuff in the footer, so find where that starts
	$footer = 0;
	find_tag($buffer, '<td valign="top"><b>icon legend:</b><br><img src', $footer, '');
	if ($footer == 0)
	{
		warning('Footer not found');
		$footer = strlen($buffer);
	}

	for ($cnt=0; find_tag($buffer, 'showMessage', $p, '') && ($p < $footer); $cnt++)
    {
        $topicID = intval(retrieve_value($buffer, 'topicID=', '.topic', $p));
        $topic = retrieve_value($buffer, 'topic>', '</a>', $p);

		// If a topic is moved, there is text after it
		$moved_test = retrieve_value($buffer, '</a>', '</td>', $p);
		if (strpos($moved_test, 'moved'))
		{
			continue;
		}

        $replies = $can_see_replies ? retrieve_value($buffer, 'class=m>', '</td>', $p) : 0;
        $views = $can_see_views ? retrieve_value($buffer, 'class=m>', '</td>', $p) : 0;
		$lastpost = parse_timestr(retrieve_value($buffer, '<nobr>', '</nobr>', $p));

		// Sticky topics have an image & alt attr. inside the link
		$sticky = (strpos($topic, 'ALT="Sticky Topic"') ? 1 : 0);

		$topic = unhtml($topic);

        notice("Checking topic '$topic'");

        $sql = "SELECT ezboard_page, topic_sticky FROM " . EZBOARD_CONVERSION . " WHERE ezboard_topic=$topicID AND forum_id=" . $config['forum_id'];
        $result = exec_sql ($sql);

        if (! $row=$db->sql_fetchrow($result))
        {
            $sql = 'INSERT INTO ' . EZBOARD_CONVERSION . ' (ezboard_topic, ezboard_page, forum_id, topic_title, topic_locked, topic_sticky, topic_poster, topic_replies, topic_views, topic_lastpost, phpbb_topic) '.
                   "VALUES ($topicID, $page, " . $config['forum_id'] . ", '".str_replace("'", "''", $topic)."', 0, $sticky, 99999999, $replies, $views, $lastpost, -1)";
            exec_sql ($sql);
            notice("Topic '$topic' added.");
        }
        else
        {
			// Added CJB - ezBoard seems to allow us to fetch page 100, for example, even if there
			// are only 6 pages; it just returns the last page again. This causes an endless loop.
			//
			// If we end up in this 'else' it means the database row already exists for this topic.
			// Check for the same topic ID found on two different pages.
			//
			// NOTE: if this works it makes the "multi page" flag obsolete (always set it to "Y").
			// Modified by JT - Retrieve topic type and check to see if it's sticky.
			// Sticky topics appear on every page and this is a good thing, a happy thing.  Yay.
			$dbpage = $row['ezboard_page'];
			$sticky = $row['topic_sticky'];
			echo ( "Parse page = '". $page. "'; db page = '". $dbpage. "'; topic type ='" . $tt . "'.<br>" );
			if ( $dbpage != "" && $dbpage != $page && !$sticky)
			{
				$cnt = 0;
				break;
			}
			// Draknek - how does $dbpage become null? Not sure how this could come up
				if ( $dbpage == "" )
			{
				echo( "<font color=\"#ff0000\">WARNING: topic '".str_replace("'", "''", $topic)."' was previously imported to forum ". $config['forum_id']. ". This may cause the import to malfunction. Please create a new forum and try again.</font><br>" );
			}
			// End added code
        }

    }

	if ($config['closeThreadImage'] == '')
	{
		// Stop an error appearing if this is blank
		return $cnt;
	}

    // scan the page again for closed topics
    $p = 0;
    while ((find_tag($buffer, $config['closeThreadImage'], $p, '')) && $i++ < 25)
    {
       if (!find_tag ($buffer, "showMessage", $p, ''))
       {
           break;
       }
       $topicID = retrieve_value($buffer, "=", ".", $p) +0;
       $sql = "UPDATE ". EZBOARD_CONVERSION. " SET topic_locked=1 WHERE ezboard_topic=$topicID AND forum_id=" . $config['forum_id'];
       exec_sql ($sql);
       $p += 10;
    }

    //echo "Found $cnt topics<br>";
    return ($cnt);
}

// parses an EZBOARD topic page
// if EZBOARD changes its html then this function might need to be recoded.
function parse_topic ($topicID, $phpbb_topic, $start)
{
    global $config;

    $cnt = 0;
    $stop = $start + 19;
    // To make the surfing more realistic...  Let's be smart about the URL.  First page is linked differently from subsequent pages.
    if ($start > 1)
    {
	    open_page (".showMessageRange?topicID=$topicID.topic&start=$start&stop=$stop", $buffer);
    }
    else
    {
	    open_page (".showMessage?topicID=$topicID.topic", $buffer);
    }

	// Skip any sidebars, etc.
    $p=0;
	find_tag($buffer, '>Author</font>', $p, '');

    while (find_tag($buffer, 'class=title', $p, ''))
    {
		$author = retrieve_value($buffer, 'title>', '</span>', $p);

		$user_registered = strpos($author, 'PublicProfile') ? TRUE : FALSE;

		$author = unhtml($author);

		$user_id = convert_user($author, $user_registered);

		if ($user_id != -1)
		{
			$author = '';
		}

		$tim = retrieve_value($buffer, '>(', ')</font>', $p);

		find_tag($buffer, 'class=title', $p, 'Can\'t find title of post.');

		$title = unhtml(retrieve_value ($buffer, 'title>', '</span', $p));

		// Modified JT - Slight (dangerous?) change here to better detect the end of post.
		// Some users have tables in their sigs.  Post detector would stop at the </td></tr> before.
		// EZBoard also puts some other weird tags like <p></p><i></i></td></tr>, so let's use those
		// to try to find the end.  Add the </i> back in so we don't leave <i> hanging.

		$text = trim(retrieve_value ($buffer, '<hr size=1>', '</i></td></tr>', $p));
		$text .= '</i>';
        if ($config['strip_html'])
        {
            $text = preg_replace('<br\s*/?\s*>',"\n", $text);
            $text = strip_tags($text, '<b><i><u>');
        }
        $timestamp=parse_timestr ($tim);
		$index = $start + $cnt - 1;
        $cnt++;
        convert_posting ($phpbb_topic, $index, $user_id, $author, $timestamp, $title, $text);
    }
    if ($cnt == 0)
    {
          $buflen = strlen ($buffer);
          warning("No postings found in topic nr $topicID. Parsed $buflen bytes.  This could mean that we mistakenly thought there was a second page.");

    }

    //echo "Converted $cnt postings<br>\n";
    return ($cnt);
}


// Converts posting time from a string to a timestamp
//
// Now uses strtotime(), which we should have been doing from the beginning
function parse_timestr ($string)
{
	global $config;

	$string = trim($string);

	if ($config['date_format'] == 'ddmm')
	{
		$string = preg_replace('!^([^/]*)/([^/]*)/!', '$2/$1/', $string);
	}

	if (is_numeric($config['timezone']))
	{
		$string .= ' GMT';
	}

	$date = strtotime($string);

	if (is_numeric($config['timezone']))
	{
		$date -= $config['timezone'] * 3600;
	}

	return $date;
}

?>
