<?php
/***************************************************************************
 *                                stage_0.php
 *                            -------------------
 *   begin                : Saturday, 19 June 2004
 *   author               : Alan Hazelden (Draknek)
 *   email                : ezconvert@draknek.org
 *
 *   purpose              : Sets up the configuration options
 *
 ***************************************************************************/

/***************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 ***************************************************************************/

if (!defined('IN_PHPBB'))
   die("Hacking attempt");

// Conversion from ezBoard to phpBB takes place in 5 stages
// Stage 0 configures the various options

$table_exists = table_exists(EZBOARD_CONVERSION);

if ($table_exists)
{
	if ($config['table_schema'] == EZCONVERT_CURRENT_TABLE_SCHEMA)
	{
		// Table is up to date, so we only need to remove the current information
		$sql = 'DELETE FROM ' . EZBOARD_CONVERSION;
		exec_sql ($sql);
		notice('Table ' . EZBOARD_CONVERSION . ' has been wiped.');
	}
	else
	{
		// Table isn't up to date - needs to be deleted so it can be recreated with all fields
		$sql = 'DROP TABLE ' . EZBOARD_CONVERSION;
		exec_sql ($sql);
		$table_exists = 0;
		notice('Table ' . EZBOARD_CONVERSION . ' has been deleted because new fields have been added and the table must be updated.');
	}
}

if (!$table_exists)
{
	// Table doesn't exist, so create it
	$sql = 'CREATE TABLE ' . EZBOARD_CONVERSION . ' ('.
		'forum_id int NOT NULL, '.
		'ezboard_topic int NOT NULL, '.
		'ezboard_page int NOT NULL, '.
		'topic_locked smallint NOT NULL, '.
		'topic_sticky smallint NOT NULL, '.
		'topic_title varchar(60) NOT NULL, '.
		'topic_poster int NOT NULL, '.
		'topic_replies int NOT NULL, '.
		'topic_views int NOT NULL, '.
		'topic_lastpost int NOT NULL, '.
		'phpbb_topic int NOT NULL )';
	exec_sql ($sql);
	notice('Table ' . EZBOARD_CONVERSION . ' has been created.');

	if ($config['table_schema'] != EZCONVERT_CURRENT_TABLE_SCHEMA)
	{
		update_config(array('table_schema' => EZCONVERT_CURRENT_TABLE_SCHEMA));
	}
}

if ($HTTP_SERVER_VARS['REQUEST_METHOD'] == 'POST')
{
    // List of variables we want to deal with
    $variables = array(
        'ezboard_forum',
        'forum_id',
        'default_pass',
        'default_pass2',
        'debug_level',
        'use_admin',
        'ezboard_user',
        'ezboard_pass',
        'connect_with',
        'strip_html',
        'enable_sigs',
        'strip_crlf',
        'closeThreadImage',
        'multiple_pages',
        'date_format',
        'timezone',
		'user_create',
    );

    foreach ($variables as $name)
    {
        $config_change[$name] = $HTTP_POST_VARS[$name];
    }

	update_config($config_change);
    notice('Your configuration changes have been made. '.
           'The settings below should have updated to reflect these changes.', true);
}

// Get the list of forums/categories for later use

$forums = array();
$sql = 'SELECT forum_id, cat_id, forum_name FROM ' . FORUMS_TABLE . ' ORDER BY forum_order';
$result = exec_sql($sql);
while ($row = $db->sql_fetchrow($result))
{
    $forums[$row['forum_id']] = $row;
    // Remember: even though forum_id is numerical and being used as a key here,
    // the order the array is traversed in still depends on the order of input
}

$categories = array();
$sql = 'SELECT cat_id FROM ' . CATEGORIES_TABLE . ' ORDER BY cat_order';
$result = exec_sql($sql);
while ($row = $db->sql_fetchrow($result))
{
    $categories[] = $row['cat_id'];
}

$errors = array();

if (!defined('EZCONVERT_CONFIGURED'))
{
    // This means the config file is blank
    $errors[] = 'Before you can start, you must set several configuration options. These are outlined below.';
}
else
{
    if (empty($config['ezboard_forum']) or $config['ezboard_forum'] == 'http://')
    {
        $errors[] = 'You have not entered an ezBoard URL';
    }
    if (empty($forums[$config['forum_id']]))
    {
        $errors[] = 'There is no phpBB forum with ID ' . $config['forum_id'];
    }
    if ($config['default_pass'] != $config['default_pass2'])
    {
        $errors[] = 'The passwords you entered did not match';
        unset($config['default_pass'],$config['default_pass2']);
    }
    elseif (empty($config['default_pass']))
    {
        $errors[] = 'You have not entered a default password for new user accounts';
    }
    if ($config['connect_with'] == 'curl' and !function_exists('curl_init'))
    {
        $errors[] = 'You have chosen to use cURL, but it is not installed.';
    }
    elseif (!(empty($config['ezboard_forum']) or $config['ezboard_forum'] == 'http://'))
    {
        open_page ('?page=1', $buffer, $false);
        if (! strpos ($buffer, 'showMessage'))
        {
            $errors[] = 'The url <a href="' . $config['ezboard_forum'] . '" target="_blank">' . $config['ezboard_forum'] . '</a> is not a valid ezboard forum. '.
                        'Make sure this url points to an individual ezboard forum instead of the board listing your forums.';
        }
    }
    if ($config['use_admin'] and (empty($config['ezboard_user'])))
    {
        $errors[] = 'You have chosen to use an administrator account to migrate this forum, but your ezBoard username is blank.';
    }
    if ($config['use_admin'] and (empty($config['ezboard_pass'])))
    {
        $errors[] = 'You have chosen to use an administrator account to migrate this forum, but your ezBoard password is blank.';
    }
    if ($config['use_admin'] and $config['ezboard_user'] and $config['ezboard_pass'])
    {
        $data = admin_login(true);
        if (strpos($data, 'Set-Cookie:') === false)
        {
            $errors[] = 'You don\'t seem able to log into ezBoard. Check <a href="' . $ezconvert_dir .
            'login.html">login.html</a> to see if there is an error. The <a href="' . $ezconvert_dir .
            'headers.txt">returned headers</a> may also be helpful.';
        }
    }
}

if (empty($errors))
{
    echo 'If you are happy with these configuration options, <a href="';
    echo append_sid('admin_ezconvert.' . $phpEx . '?stage=1');
    echo '">click here</a> to begin the conversion.';
}
else
{
    echo 'The following errors were detected, which much be corrected before you can continue:';
    echo "\n<ul>\n";
    foreach ($errors as $error)
        echo '<li>' . $error . "</li>\n";
    echo "</ul>\n";
}

include($ezconvert_dir . 'form_setup.' . $phpEx);

if (empty($errors))
{
    echo 'If you are happy with these configuration options, <a href="';
    echo append_sid('admin_ezconvert.' . $phpEx . '?stage=1');
    echo '">click here</a> to begin the conversion.';
}

?>
